from strenum import StrEnum

class DAO(StrEnum):
    OWNERSHIP = "ownership"
    PARAM = "param"
    EMERGENCY = "emergency"


CURVE_DAO_OWNERSHIP = {
    "agent": "0x40907540d8a6C65c637785e8f8B742ae6b0b9968",
    "voting": "0xE478de485ad2fe566d49342Cbd03E49ed7DB3356",
    "token": "0x5f3b5DfEb7B28CDbD7FAba78963EE202a494e2A2",
    "quorum": 30,
}

CURVE_DAO_PARAM = {
    "agent": "0x4eeb3ba4f221ca16ed4a0cc7254e2e32df948c5f",
    "voting": "0xbcff8b0b9419b9a88c44546519b1e909cf330399",
    "token": "0x5f3b5DfEb7B28CDbD7FAba78963EE202a494e2A2",
    "quorum": 15,
}

EMERGENCY_DAO = {
    "agent": "0x467947EE34aF926cF1DCac093870f613C96B1E0c",
}
GAUGE_CONTROLLER = "0x2F50D538606Fa9EDD2B11E2446BEb18C9D5846bB"
VOTING_ESCROW = "0x5f3b5DfEb7B28CDbD7FAba78963EE202a494e2A2"
veCRV = VOTING_ESCROW
CRV = "0xD533a949740bb3306d119CC777fa900bA034cd52"
CONVEX_VOTERPROXY = "0x989AEB4D175E16225E39E87D0D97A3360524AD80"
ADDRESSPROVIDER = "0x5ffe7FB82894076ECB99A30D6A32e969e6e35E98"  # On all chains


def get_address(vote_type: DAO | str):
    """ Get execution address of DAO (e.g. admin role)"""
    return get_dao_parameters(vote_type)["agent"]


def get_dao_parameters(vote_type: DAO | str):
    """
    :param vote_type: Name, agent address or enum
    """
    if vote_type.startswith("0x"):
        # {agent address: enum}
        vote_type = {get_dao_parameters(dao)["agent"]: dao for dao in DAO}[vote_type]
    match DAO(vote_type):
        case DAO.OWNERSHIP:
            return CURVE_DAO_OWNERSHIP
        case DAO.PARAM:
            return CURVE_DAO_PARAM
        case DAO.EMERGENCY:
            return EMERGENCY_DAO



import pytest

import curve_dao


def test_get_address():
    assert curve_dao.get_address("ownership") == curve_dao.get_address(
        curve_dao.addresses.DAO.OWNERSHIP
    )
    assert curve_dao.get_address("param") == curve_dao.get_address(
        curve_dao.addresses.DAO.PARAM
    )
    assert curve_dao.get_address("emergency") == curve_dao.get_address(
        curve_dao.addresses.DAO.EMERGENCY
    )


@pytest.mark.parametrize(
    "target",
    [
        curve_dao.addresses.DAO.OWNERSHIP,
        "param",
        "0x467947EE34aF926cF1DCac093870f613C96B1E0c",
    ],
)
def test_target(target, etherscan_api_key):
    curve_dao.prepare_evm_script(target, [], etherscan_api_key)



import logging
import warnings
from typing import List, Tuple

import boa
from boa.contracts.abi.abi_contract import ABIContract
from boa.contracts.vyper.vyper_contract import VyperContract
from hexbytes import HexBytes
from rich.logging import RichHandler

from curve_dao.addresses import DAO, get_dao_parameters

warnings.filterwarnings("ignore")

logging.basicConfig(
    level="INFO",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True)],
)

logger = logging.getLogger("rich")


class MissingVote(Exception):
    """Exception raised when a vote ID is invalid."""


def prepare_evm_script(dao: str | DAO, actions: List[Tuple], etherscan_api_key: str):
    """Generates EVM script to be executed by AragonDAO contracts.

    Args:
        target (dict): CURVE_DAO_OWNERSHIP / CURVE_DAO_PARAMS / EMERGENCY_DAO
        actions (list(tuple)): ("target addr", "fn_name", *args)

    Returns:
        str: Generated EVM script.
    """
    aragon_agent = boa.from_etherscan(
        get_dao_parameters(dao)["agent"], name="AragonAgent", api_key=etherscan_api_key
    )

    logger.info("Preparing EVM script")
    evm_script = bytes.fromhex("00000001")

    for action in actions:
        address, fn_name, *args = action
        contract = (
            address
            if isinstance(address, (VyperContract, ABIContract))
            else boa.from_etherscan(
                address=address, name="TargetContract", api_key=etherscan_api_key
            )
        )
        contract_function = getattr(contract, fn_name)

        calldata = contract_function.prepare_calldata(*args)
        agent_calldata = aragon_agent.execute.prepare_calldata(address, 0, calldata)

        length = bytes.fromhex(hex(len(agent_calldata.hex()) // 2)[2:].zfill(8))
        evm_script = (
            evm_script
            + bytes.fromhex(aragon_agent.address[2:])
            + length
            + agent_calldata
        )

    evm_script = HexBytes(evm_script)
    logger.info(f"EVM script generated: {evm_script}")

    return evm_script



import logging
from typing import List, Tuple

import boa
from hexbytes import HexBytes
from rich.logging import RichHandler

from .addresses import DAO, get_address, get_dao_parameters  # noqa: F401
from .ipfs import pin_to_ipfs  # noqa: F401
from .simulate import simulate  # noqa: F401
from .vote_utils import prepare_evm_script  # noqa: F401

logging.basicConfig(
    level="INFO",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True)],
)

logger = logging.getLogger("rich")


def create_vote(
    dao: str | DAO,
    actions: List[Tuple],
    description: str,
    etherscan_api_key: str,
    pinata_token: str,
) -> int:
    evm_script = prepare_evm_script(dao, actions, etherscan_api_key)
    ipfs_hash_of_description = pin_to_ipfs(description, pinata_token)

    voting = boa.from_etherscan(
        get_dao_parameters(dao)["voting"],
        name="AragonVoting",
        api_key=etherscan_api_key,
    )

    try:
        assert voting.canCreateNewVote(boa.env.eoa)
    except Exception:
        logger.exception(
            "EOA cannot create new vote. Either there isn't enough veCRV balance or"
            "EOA created a vote less than 12 hours ago."
        )

    return voting.newVote(
        HexBytes(evm_script), f"ipfs:{ipfs_hash_of_description}", False, False
    )
